@@ -38,14 +38,6 @@ $(document).ready(function () {
   // make list of graphs draggable
   $('#draggablelist').sortable({
     handle: '.draghandle',
-    start: function () {
-      // hide and disable all tooltips
-      $('.tooltip').tooltip('hide').tooltip('disable')
-    },
-    stop: function () {
-      // enable tooltips again
-      $('.tooltip').tooltip('enable')
-    },
     update: function (event, ui) {
       // after any changes, save the current list of graphs
       saveCurrentGraphs()
@@ -189,9 +181,9 @@ $(document).ready(function () {
     Object.keys(plot.legendbyfield).forEach(function (field) {
       // calculate minimum
       const mintrace = plot.tracebyfield[field + '.min']
-      const minvalue = Math.min.apply(null, mintrace.y.filter(function (el, idx) {
+      const minvalue = Math.min.apply(null, mintrace.y.filter(function (y, idx) {
         const x = mintrace.x[idx]
-        return x >= minx && x <= maxx
+        return x >= minx && x <= maxx && y !== null
       }))
       // calculate average
       const avgtrace = plot.tracebyfield[field]
@@ -213,26 +205,158 @@ $(document).ready(function () {
       }
       // calculate maximum
       const maxtrace = plot.tracebyfield[field + '.max']
-      const maxvalue = Math.max.apply(null, maxtrace.y.filter(function (el, idx) {
+      const maxvalue = Math.max.apply(null, maxtrace.y.filter(function (y, idx) {
         const x = maxtrace.x[idx]
-        return x >= minx && x <= maxx
+        return x >= minx && x <= maxx && y !== null
       }))
       // update legend
       const columns = $(plot.legendbyfield[field]).find('td')
-      columns[2].textContent = (isNaN(minvalue) || !isFinite(minvalue)) ? '-' : Plotly.d3.format('.4s')(minvalue)
-      columns[3].textContent = (isNaN(avgvalue) || !isFinite(avgvalue)) ? '-' : Plotly.d3.format('.4s')(avgvalue)
-      columns[4].textContent = (isNaN(maxvalue) || !isFinite(maxvalue)) ? '-' : Plotly.d3.format('.4s')(maxvalue)
+      columns[2].textContent = (isNaN(minvalue) || !isFinite(minvalue)) ? '-' : d3.format('.4s')(minvalue)
+      columns[3].textContent = (isNaN(avgvalue) || !isFinite(avgvalue)) ? '-' : d3.format('.4s')(avgvalue)
+      columns[4].textContent = (isNaN(maxvalue) || !isFinite(maxvalue)) ? '-' : d3.format('.4s')(maxvalue)
     })
   }
 
   // load graph data into the plot
-  function loadGraph(plot, legend, graph) {
+  function loadGraph(plot, traces, layout) {
+    // range of the x axis
+    let url = 'data/' + plot.graph.name
+    if (plot.layout) {
+      const [amin, amax] = plot.layout.xaxis.range
+      url += '?start=' + amin.substring(0, 16).replace(' ', 'T') + '&end=' + amax.substring(0, 16).replace(' ', 'T')
+    }
+    d3.csv(url).then(function (data) {
+      // clear traces
+      Object.keys(plot.tracebyfield).forEach(function (field) {
+        plot.tracebyfield[field].x = []
+        plot.tracebyfield[field].y = []
+      })
+      // load new data
+      data.forEach(function (row) {
+        Object.keys(plot.tracebyfield).forEach(function (field) {
+          const value = row[field] ? Number(row[field]) : null
+          if (row[field] || plot.tracebyfield[field].y.length > 0) {
+            plot.tracebyfield[field].x.push(row.time)
+            plot.tracebyfield[field].y.push(value)
+          }
+        })
+      })
+      // remove trailing nulls from traces
+      Object.keys(plot.tracebyfield).forEach(function (field) {
+        while (plot.tracebyfield[field].y[plot.tracebyfield[field].y.length - 1] === null) {
+          plot.tracebyfield[field].x.pop()
+          plot.tracebyfield[field].y.pop()
+        }
+      })
+      if (traces) {
+        // initial plot creation
+        plot.innerHTML = ''
+        Plotly.react(plot, traces, layout, config)
+        updateLegend(plot)
+        updatedata = true
+        // handle plot zoom changes
+        plot.on('plotly_relayout', function (data) {
+          if (data['xaxis.range[0]'] && data['xaxis.range[1]']) {
+            updatedata = true
+            setDateRange(data['xaxis.range[0]'], data['xaxis.range[1]'])
+            // update the legend values
+            updateLegend(plot)
+          }
+        })
+        // after any changes, save the current list of graphs
+        saveCurrentGraphs()
+      } else {
+        plot.layout.datarevision += 1
+        Plotly.react(plot, plot.data, plot.layout)
+      }
+    })
+  }
+
+  // check if the axis match the data range and load more data as needed
+  function checkDataUpdates() {
+    try {
+      if (updatedata) {
+        updatedata = false
+        // go over all plots
+        $('.myplot').each(function () {
+          const plot = this
+          if (plot.layout) {
+            loadGraph(plot)
+          }
+        })
+      }
+    } finally {
+      setTimeout(checkDataUpdates, 1000)
+    }
+  }
+  setTimeout(checkDataUpdates, 1000)
+
+  // every minute check if we should load new data
+  function checkNewData() {
+    setTimeout(checkNewData, 60000)
+    $('.myplot').each(function (index, plot) {
+      // if any plot has incomplete data, reload the data
+      if (plot.layout) {
+        if (moment().format('YYYY-MM-DD HH:mm') < plot.layout.xaxis.range[1]) { updatedata = true }
+      }
+    })
+  }
+  setTimeout(checkNewData, 60000)
+
+  function addGraph(graph, size = 'sm') {
+    const clone = $('#template>:first-child').clone()
+    const plot = clone.find('.myplot')[0]
+    const legend = clone.find('.mylegend')
+    plot.graph = graph
+    // update graph title
+    clone.find('.graphtitle').append($('<div>').addClass('col').text(graph.host + ' / ')
+      .append($('<b>').text(graph.graph_title))
+      .attr('title', graph.graph_info || ''))
+    // set the size changing actions
+    clone.find('.sizesm').click(function () {
+      clone.find('.sizeactive').removeClass('sizeactive')
+      $(this).addClass('sizeactive')
+      $(plot).addClass('plot-sm').removeClass('plot-md plot-lg')
+      legend.addClass('legend-sm').removeClass('legend-md legend-lg')
+      Plotly.relayout(plot, {})
+      saveCurrentGraphs()
+    })
+    clone.find('.sizemd').click(function () {
+      clone.find('.sizeactive').removeClass('sizeactive')
+      $(this).addClass('sizeactive')
+      $(plot).addClass('plot-md').removeClass('plot-sm plot-lg')
+      legend.addClass('legend-md').removeClass('legend-sm legend-lg')
+      Plotly.relayout(plot, {})
+      saveCurrentGraphs()
+    })
+    clone.find('.sizelg').click(function () {
+      clone.find('.sizeactive').removeClass('sizeactive')
+      $(this).addClass('sizeactive')
+      $(plot).addClass('plot-lg').removeClass('plot-sm plot-md')
+      legend.addClass('legend-lg').removeClass('legend-sm legend-md')
+      Plotly.relayout(plot, {})
+      saveCurrentGraphs()
+    })
+    // configure the close button
+    clone.find('.closegraph').click(function () {
+      clone.hide(400, function () {
+        Plotly.purge(plot)
+        $(this).remove()
+        // after any changes, save the current list of graphs
+        saveCurrentGraphs()
+      })
+    })
+    // set the wanted size
+    $(plot).addClass('plot-' + size)
+    legend.addClass('legend-' + size)
+    clone.find('.sizeactive').removeClass('sizeactive')
+    clone.find('.size' + size).addClass('sizeactive')
     // prepare the graph configuration
     const layout = JSON.parse(JSON.stringify(baseLayout))
-    if (graph.graph_vlabel) {
-      layout.yaxis.title = graph.graph_vlabel
+    if (plot.graph.graph_vlabel) {
+      layout.yaxis.title = plot.graph.graph_vlabel
     }
-    if (graph.graph_args && graph.graph_args.match(/--logarithmic/)) {
+    if (plot.graph.graph_args && plot.graph.graph_args.match(/--logarithmic/)) {
       layout.yaxis.type = 'log'
       layout.yaxis.exponentformat = 'E'
     }
@@ -246,11 +370,11 @@ $(document).ready(function () {
     const tracebyfield = {}
     plot.tracebyfield = tracebyfield
     let stackgroup = 0
-    for (let i = 0; i < graph.fields.length; i++) {
-      const field = graph.fields[i]
+    for (let i = 0; i < plot.graph.fields.length; i++) {
+      const field = plot.graph.fields[i]
       const color = field.colour ? '#' + field.colour : defaultColors[i % defaultColors.length]
       if (field.draw === 'AREA' || field.draw === 'STACK' || field.draw === 'AREASTACK') {
-        if (!field.draw.match(/STACK/) && (!graph.fields[i + 1] || graph.fields[i + 1].draw.match(/STACK/))) {
+        if (!field.draw.match(/STACK/) && (!plot.graph.fields[i + 1] || plot.graph.fields[i + 1].draw.match(/STACK/))) {
           stackgroup += 1
         }
         const trace = {
@@ -278,13 +402,15 @@ $(document).ready(function () {
           field_name: field.name,
           showlegend: false,
           hoverinfo: 'skip',
-          line: {width: 0}
+          line: {width: 0},
+          connectgaps: true
         }
         const maxTrace = {
           field_name: field.name,
           showlegend: false,
           hoverinfo: 'skip',
           line: {width: 0},
+          connectgaps: true,
           fill: 'tonexty',
           fillcolor: color + '20'
         }
@@ -363,136 +489,7 @@ $(document).ready(function () {
       }
     })
     // fetch the data and plot it
-    // TODO: probably skip this initial load???
-    Plotly.d3.csv('data/' + graph.name, function (data) {
-      for (let i = 0; i < data.length; i++) {
-        const row = data[i]
-        Object.keys(tracebyfield).forEach(function (field) {
-          tracebyfield[field].x.push(row.time)
-          tracebyfield[field].y.push(Number(row[field]))
-        })
-      }
-      plot.innerHTML = ''
-      Plotly.react(plot, traces, layout, config)
-      updateLegend(plot)
-      updatedata = true
-      // handle plot zoom changes
-      plot.on('plotly_relayout', function (data) {
-        if (data['xaxis.range[0]'] && data['xaxis.range[1]']) {
-          updatedata = true
-          setDateRange(data['xaxis.range[0]'], data['xaxis.range[1]'])
-          // update the legend values
-          updateLegend(plot)
-        }
-      })
-      // after any changes, save the current list of graphs
-      saveCurrentGraphs()
-    })
-  }
-
-  // check if the axis match the data range and load more data as needed
-  function checkDataUpdates() {
-    try {
-      if (updatedata) {
-        updatedata = false
-        // go over all plots
-        $('.myplot').each(function () {
-          const plot = this
-          if (plot.layout) {
-            // range of the x axis
-            const [amin, amax] = plot.layout.xaxis.range
-            const url = 'data/' + plot.graph.name + '?start=' + amin.substring(0, 16).replace(' ', 'T') + '&end=' + amax.substring(0, 16).replace(' ', 'T')
-            Plotly.d3.csv(url, function (data) {
-              Object.keys(plot.tracebyfield).forEach(function (field) {
-                plot.tracebyfield[field].x = []
-                plot.tracebyfield[field].y = []
-              })
-              data.forEach(function (row) {
-                Object.keys(plot.tracebyfield).forEach(function (field) {
-                  plot.tracebyfield[field].x.push(row.time)
-                  plot.tracebyfield[field].y.push(Number(row[field]))
-                })
-              })
-              plot.layout.datarevision += 1
-              Plotly.react(plot, plot.data, plot.layout)
-            })
-          }
-        })
-      }
-    } finally {
-      setTimeout(checkDataUpdates, 1000)
-    }
-  }
-  setTimeout(checkDataUpdates, 1000)
-
-  // every minute check if there is any new data
-  function checkNewData() {
-    setTimeout(checkNewData, 60000)
-    $('.myplot').each(function (index, plot) {
-      // if any plot has incomplete data, reload the data
-      if (plot.layout) {
-        const dmax = plot.data.map(t => t.x[t.x.length - 1]).reduce((a, c) => a > c ? a : c)
-        if (dmax < plot.layout.xaxis.range[1]) { updatedata = true }
-      }
-    })
-  }
-  setTimeout(checkNewData, 60000)
-
-  function addGraph(graph, size = 'sm') {
-    const clone = $('#template>:first-child').clone()
-    const plot = clone.find('.myplot')[0]
-    const legend = clone.find('.mylegend')
-    plot.graph = graph
-    // update graph title
-    clone.find('.graphtitle').text(graph.host + ' / ')
-      .append($('<b>').text(graph.graph_title))
-      .tooltip({title: graph.graph_info || ''})
-    // tooltip for drag handle
-    clone.find('.draghandle').tooltip({placement: 'right'})
-    // set the size changing actions
-    clone.find('.sizesm').tooltip({placement: 'right'}).click(function () {
-      clone.find('.sizeactive').removeClass('sizeactive')
-      $(this).addClass('sizeactive')
-      $(plot).addClass('plot-sm').removeClass('plot-md plot-lg')
-      legend.addClass('legend-sm').removeClass('legend-md legend-lg')
-      Plotly.relayout(plot, {})
-      saveCurrentGraphs()
-    })
-    clone.find('.sizemd').tooltip({placement: 'right'}).click(function () {
-      clone.find('.sizeactive').removeClass('sizeactive')
-      $(this).addClass('sizeactive')
-      $(plot).addClass('plot-md').removeClass('plot-sm plot-lg')
-      legend.addClass('legend-md').removeClass('legend-sm legend-lg')
-      Plotly.relayout(plot, {})
-      saveCurrentGraphs()
-    })
-    clone.find('.sizelg').tooltip({placement: 'right'}).click(function () {
-      clone.find('.sizeactive').removeClass('sizeactive')
-      $(this).addClass('sizeactive')
-      $(plot).addClass('plot-lg').removeClass('plot-sm plot-md')
-      legend.addClass('legend-lg').removeClass('legend-sm legend-md')
-      Plotly.relayout(plot, {})
-      saveCurrentGraphs()
-    })
-    // configure the close button
-    clone.find('.closegraph').tooltip({placement: 'right'}).click(function () {
-      $(this).tooltip('dispose')
-      clone.hide(400, function () {
-        Plotly.purge(plot)
-        $(this).remove()
-        // after any changes, save the current list of graphs
-        saveCurrentGraphs()
-      })
-    })
-    // set the wanted size
-    $(plot).addClass('plot-' + size)
-    legend.addClass('legend-' + size)
-    clone.find('.sizeactive').removeClass('sizeactive')
-    clone.find('.size' + size).addClass('sizeactive')
-    // load the graph data
-    loadGraph(plot, legend, graph)
-    // enable tooltips on legend
-    clone.find('.mylegend *[title]').tooltip({placement: 'bottom', container: 'body'})
+    loadGraph(plot, traces, layout)
     // show the graph
     clone.appendTo('#draggablelist')
     return plot
@@ -540,12 +537,12 @@ $(document).ready(function () {
         if (categoryFilter && graphs[graph].category !== categoryFilter) {
           return
         }
-        const descripton = (graph + ' ' + graphs[graph].graph_title + ' ' + graphs[graph].category).toLowerCase()
-        if (search.some(x => !descripton.includes(x))) {
+        const description = (graph + ' ' + graphs[graph].graph_title + ' ' + graphs[graph].category).toLowerCase()
+        if (search.some(x => !description.includes(x))) {
           return
         }
         const title = graphs[graph].graph_title || graph.split('/')[2]
-        const graphelement = $('<a href="#" class="list-group-item list-group-item-action" data-toggle="collapse" data-target="#addgraph"></a>').text(title)
+        const graphelement = $('<a href="#" class="list-group-item list-group-item-action" data-bs-toggle="collapse" data-bs-target="#addgraph"></a>').text(title)
         // add the host graph unless a host graph has been selected
         if (!hostFilter) {
           graphelement.prepend($('<small></small>').text(graph.split('/')[1] + ' / '))
@@ -679,10 +676,10 @@ $(document).ready(function () {
   })
 
   // handle showing and hiding of the save dashboard dialog
-  $('#saveDashboard').on('shown.bs.modal', function () {
+  document.getElementById('saveDashboard').addEventListener('shown.bs.modal', function () {
     $('#saveDashboardName').trigger('focus')
   })
-  $('#saveDashboard').on('hidden.bs.modal', function () {
+  document.getElementById('saveDashboard').addEventListener('hidden.bs.modal', function () {
     $('#saveDashboardName').val('')
     $('#saveDashboard .alert').remove()
   })
@@ -723,7 +720,7 @@ $(document).ready(function () {
     // show notification
     $('<div class="alert alert-success">Copied to clipboard</div>').hide().appendTo('#saveDashboard .modal-body').show(200, function () {
       setTimeout(function () {
-        $('#saveDashboard').modal('hide')
+        bootstrap.Modal.getInstance(document.getElementById('saveDashboard')).hide()
       }, 600)
     })
   })
@@ -737,14 +734,14 @@ $(document).ready(function () {
     document.body.appendChild(element)
     element.click()
     document.body.removeChild(element)
-    $('#saveDashboard').modal('hide')
+    bootstrap.Modal.getInstance(document.getElementById('saveDashboard')).hide()
   })
 
   // handle showing and hiding of the load dashboard dialog
-  $('#loadDashboard').on('shown.bs.modal', function () {
+  document.getElementById('loadDashboard').addEventListener('shown.bs.modal', function () {
     $('#loadDashboardData').trigger('focus')
   })
-  $('#loadDashboard').on('hidden.bs.modal', function () {
+  document.getElementById('loadDashboard').addEventListener('hidden.bs.modal', function () {
     $('#loadDashboardData').val('')
     $('#loadDashboard .alert').remove()
   })
@@ -774,7 +771,7 @@ $(document).ready(function () {
         if (dashboard.name) {
           $('#dashboards button.dropdown-toggle span').text(dashboard.name)
         }
-        $('#loadDashboard').modal('hide')
+        bootstrap.Modal.getInstance(document.getElementById('loadDashboard')).hide()
       }
     } catch (e) {
       $('#loadDashboard .alert').remove()
